package cn.ibizlab.core.lite.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.lite.domain.TableSync;
import cn.ibizlab.core.lite.filter.TableSyncSearchContext;
import cn.ibizlab.core.lite.service.ITableSyncService;

import cn.ibizlab.util.helper.CachedBeanCopier;
import cn.ibizlab.util.helper.DEFieldCacheMap;
import com.alibaba.fastjson.JSONObject;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.lite.mapper.TableSyncMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import org.springframework.util.StringUtils;

/**
 * 实体[ETL表同步] 服务对象接口实现
 */
@Slf4j
@Service("TableSyncServiceImpl")
public class TableSyncServiceImpl extends ServiceImpl<TableSyncMapper, TableSync> implements ITableSyncService {

    @Autowired
    @Lazy
    protected cn.ibizlab.core.lite.service.IMetaEntityService metaentityService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(TableSync et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getId()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<TableSync> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(TableSync et) {
        fillParentData(et);
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("syncid", et.getId()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getId()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<TableSync> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public TableSync get(String key) {
        TableSync et = getById(key);
        if (et == null) {
            throw new BadRequestAlertException("数据不存在", this.getClass().getSimpleName(), key);
        }
        else {
        }
        return et;
    }

    @Override
    public TableSync getDraft(TableSync et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(TableSync et) {
        return (!ObjectUtils.isEmpty(et.getId())) && (!Objects.isNull(this.getById(et.getId())));
    }
    @Override
    @Transactional
    public boolean save(TableSync et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(TableSync et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? getProxyService().update(et) : getProxyService().create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<TableSync> list) {
        list.forEach(item->fillParentData(item));
        List<TableSync> create = new ArrayList<>();
        List<TableSync> update = new ArrayList<>();
        for (TableSync et : list) {
            if (ObjectUtils.isEmpty(et.getId()) || ObjectUtils.isEmpty(getById(et.getId()))) {
                create.add(et);
            } else {
                update.add(et);
            }
        }
        if (create.size() > 0) {
            getProxyService().createBatch(create);
        }
        if (update.size() > 0) {
            getProxyService().updateBatch(update);
        }
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<TableSync> list) {
        list.forEach(item->fillParentData(item));
        List<TableSync> create = new ArrayList<>();
        List<TableSync> update = new ArrayList<>();
        for (TableSync et : list) {
            if (ObjectUtils.isEmpty(et.getId()) || ObjectUtils.isEmpty(getById(et.getId()))) {
                create.add(et);
            } else {
                update.add(et);
            }
        }
        if (create.size() > 0) {
            getProxyService().createBatch(create);
        }
        if (update.size() > 0) {
            getProxyService().updateBatch(update);
        }
    }

    @Override
    public TableSync sync(TableSync et) {
        //自定义代码
        return et;
    }

    @Override
    public boolean syncBatch(List<TableSync> etList) {
        for(TableSync et : etList) {
            sync(et);
        }
        return true;
    }


	@Override
    public List<TableSync> selectByEntityId(String entity_id) {
        return baseMapper.selectByEntityId(entity_id);
    }
    @Override
    public void removeByEntityId(String entity_id) {
        this.remove(new QueryWrapper<TableSync>().eq("entityid",entity_id));
    }

	@Override
    public List<TableSync> selectBySourceEntityId(String entity_id) {
        return baseMapper.selectBySourceEntityId(entity_id);
    }
    @Override
    public void removeBySourceEntityId(String entity_id) {
        this.remove(new QueryWrapper<TableSync>().eq("srcentityid",entity_id));
    }


    /**
     * 查询集合 数据集
     */
    @Override
    public Page<TableSync> searchDefault(TableSyncSearchContext context) {
            com.baomidou.mybatisplus.extension.plugins.pagination.Page<TableSync> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
            return new PageImpl<TableSync>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    public void fillParentData(TableSync et){
        //实体关系[DER1N_ETL_TABLE_SYNC_META_ENTITY_ENTITYID]
        if(!ObjectUtils.isEmpty(et.getEntityId())){
            cn.ibizlab.core.lite.domain.MetaEntity entity=et.getEntity();
            if(ObjectUtils.isEmpty(entity)){
                cn.ibizlab.core.lite.domain.MetaEntity majorEntity=metaentityService.getById(et.getEntityId());
                et.setEntity(majorEntity);
                entity=majorEntity;
            }
            et.setEntityName(entity.getEntityName());
            et.setSystemId(entity.getSystemId());
        }
        //实体关系[DER1N_ETL_TABLE_SYNC_META_ENTITY_SRCENTITYID]
        if(!ObjectUtils.isEmpty(et.getSourceEntityId())){
            cn.ibizlab.core.lite.domain.MetaEntity sourceentity=et.getSourceentity();
            if(ObjectUtils.isEmpty(sourceentity)){
                cn.ibizlab.core.lite.domain.MetaEntity majorEntity=metaentityService.getById(et.getSourceEntityId());
                et.setSourceentity(majorEntity);
                sourceentity=majorEntity;
            }
            et.setSourceEntityName(sourceentity.getEntityName());
            et.setSourceSystemId(sourceentity.getSystemId());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }





    public ITableSyncService getProxyService() {
        return cn.ibizlab.util.security.SpringContextHolder.getBean(this.getClass());
    }
}



